# -*- coding: utf-8 -*-

from os import getenv
import typepy


def FillVars(variables, v, prefix=""):
    for var in variables:
        v[var.name] = var.getenv(prefix=prefix)


class Variable(object):
    _strict_level = 2  # for typepy

    def __init__(self, name, default=None, prefix=""):
        self._real_name = name
        self.default = default
        self.prefix = prefix

    def getenv(self, prefix=""):
        value = getenv(
            self._real_name, self.default if self.default is not None else None
        )
        if value is None:
            raise ValueError(
                "Please set required '{}' variable".format(self._real_name)
            )

        self.name = self._strip_prefix(self._real_name, self.prefix or prefix)

        try:
            return self._value_type(value, strict_level=self._strict_level).convert()
        except typepy._error.TypeConversionError:
            raise ValueError(
                "Can't convert variable '{}' with value '{}' to type '{}'".format(
                    self._real_name, value, repr(self._value_type)
                )
            )

    def _strip_prefix(self, name, prefix):
        if prefix and name.startswith(prefix):
            return name[len(prefix) :]
        else:
            return name


class EnvListConverter(typepy.converter.ListConverter):
    def force_convert(self):
        try:
            return list(
                (x.strip() for x in (self._value.split(",")) if x and x.strip())
            )
        except (TypeError, ValueError):
            raise TypeConversionError(
                "failed to force_convert to list: type={}".format(type(self._value))
            )


class StringEnvListVariable(typepy.type.List):
    def _create_type_checker(self):
        return typepy.checker.StringTypeChecker(self._data, self._strict_level)

    def _create_type_converter(self):
        return EnvListConverter(self._data)


class BoolVariable(Variable):
    _value_type = typepy.type.Bool
    _strict_level = 1


class StringVariable(Variable):
    _value_type = typepy.type.String
    _strict_level = 0


class StringListVariable(Variable):
    """
        Gets a string with vars separated by comma and turns them to list
    """

    _value_type = StringEnvListVariable
    _strict_level = 0



# -*- coding: utf-8 -*-

"""Top-level package for Typed Environment Configuration."""

__author__ = """Eugene Dementyev"""
__email__ = "eugene@springload.co.nz"
__version__ = "0.1.0"

from .typed_environment_configuration import *



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `typed_environment_configuration` package."""


import unittest
import os

from typed_environment_configuration import *

_ENVVARS = [BoolVariable("DEBUG"), StringVariable("DEFAULT_STRING", default="")]

_PREFIXED_ENVVARS = [
    StringVariable("PREFIXED_VERSION"),
    StringListVariable("PREFIXED_DEFAULT_LIST", default=""),
]

v = vars()

class TestTyped_environment_configuration(unittest.TestCase):
    """Tests for `typed_environment_configuration` package."""

    def setUp(self):
        """Set up test fixtures, if any."""
        os.environ["DEBUG"] = "True"
        os.environ["PREFIXED_VERSION"] = "1.2.3"

        FillVars(_ENVVARS, v)
        FillVars(_PREFIXED_ENVVARS, v, prefix="PREFIXED_")

    def tearDown(self):
        """Tear down test fixtures, if any."""

    def test_BoolVariable(self):
        """Tests that setting boolean variable in env creates a python variable DEBUG=True"""
        self.assertEqual(DEBUG, True)

    def test_PrefixedVariable(self):
        """Tests a prefixed variable"""
        self.assertEqual(VERSION, "1.2.3")

    def test_DefaultVariable(self):
        """Tests a default variable"""
        self.assertEqual(DEFAULT_LIST, [])
